clear all
close all
clc

%% COEFF. DI PORTANZA (CL)

dati=load('dati_profilo.txt');
alpha=dati(2:end,1);
alpha=[alpha;alpha(end)];
CL=dati(2:end,2);
CL=[CL;CL(end)];

figure()
plot(alpha,CL,'LineWidth',2)
grid on
title('CL VS ALPHA')
xlabel('ALPHA')
ylabel('CL')
hold on
plot(alpha([33:110]),CL([33:110]),'ro','LineWidth',1) % USIAMO SOLO LA PARTE LINEALE, IPOTIZZIAMO PER ALPHA DA -10�� A 10��

%% DATI DI FUNZIONAMENTO E GEOMETRICI MODELLO/PROTOTIPO

u=[0.1:0.1:210]; %[m/s] VELOCITA DEL VENTO
ro_aria=1.225; %[kg/m^3] DENSITA DEL VENTO
L=7.5; %[m] LUNGHEZZA SINGOLA ALA PROTOTIPO
C_media=2;  %[m] VALORE DELLA CORDA MEDIA 
b=C_media/2; %[m]
LL=2*pi*ro_aria*b*u.^2; %PORTANZA
e=0.05*C_media; %[m] ESPESSORE PROFILLO 
S=L*C_media; %[m^2] SUPERFICIE ALARE
v=C_media*e*L; %[m^3] VOLUME PROFILLO
xa=0.1; %POSIZIONE DEL CG RISPETTO AD EA = 20%b 
a=-0.1; %POSIZIONE DELL'EA RISPETTO A MET�� DI CORDA = 40%b
cp=0.25; %POSIZIONE DEL CP RISPETTO ALL BORDO DI ATTACO = 25%C_media
x_cg=(xa+1+a)/2; %POSIZIONE DEL CG RISPETTO ALL BORDO DI ATTACO = ..%C_media
x_ea=(1+a)/2; %POSIZIONE DEL EA RISPETTO ALL BORDO DI ATTACO = ..%C_media
d=b*a+C_media/4; %DISTANZA TRA EA E CP

% DATI INERZIALI

mo=200; %[kg/m^2]
m=mo*S;
Ig=C_media^3*mo*((x_cg^3+(1-x_cg)^3)/3+((1-x_cg)^4-x_cg^4)/4); % [kg.m^2] MOMENTO D'INERZIA RISPETTO AL CG 
I_ea=Ig+m*(xa*b)^2; % [kg.m^2] MOMENTO D'INERZIA RISPETTO ALL'EA 
Sa=m*xa*b; %[Kg.m] MOMENTO STATICO 

% DATI ELASTICI

kt=(2e6)/L; %[N.m/rad] RIGIDEZZA TORSIONALE 
kl=4*(2e7)/L^3; %[N/m] RIGIDEZZA FLESSIONALE 

%% SISTEMA EQUAZIONI

wh=sqrt(kl/m);
walp=sqrt(kt/I_ea);
miu=m/(pi*ro_aria*b^2);
r=sqrt(I_ea/(m*b^2));
sigma=wh/walp;
M_aer=-1.2; 

%% CALCOLO SENZA SMORZAMENTO

M=[m*b^2 Sa*b;Sa*b I_ea]; %MATRICE DI MASSA

for i=1:length(u)
    K=[m*b^2*wh^2 LL(i)*b;0 I_ea*walp^2-LL(i)*b*(0.5+a)-LL(i)*d]; %MATRICE DI RIGIDEZZA
    C=zeros(2,2);
    MA=[C M; eye(2,2) zeros(2,2)];
    MB=[K zeros(2,2); zeros(2,2) -eye(2,2)];
    [vett val]=eig(MB,-MA);
    w1(i)=imag(val(1,1)); %AUTOVALORI
    w2(i)=imag(val(3,3));
    z1(i)=imag(val(1,1))/2*real(val(1,1));
    z2(i)=imag(val(3,3))/2*real(val(3,3));

end

figure()
subplot(3,1,1)
plot(u,w1,'LineWidth',1.5)
title('COMPORTAMENTO AERODINAMICO SENZA SMORZAMENTO')
hold on
plot(u,w2,'LineWidth',1.5)
grid on
ylabel('w [rad/s]')
axis([0 210 0 50])

subplot(3,1,2)
plot(u,w1/(2*pi),'LineWidth',1.5)
hold on
plot(u,w2/(2*pi),'LineWidth',1.5)
grid on
ylabel('Frequenza [Hz]')
axis([0 210 0 8])

subplot(3,1,3)
plot(u,z1,'LineWidth',1.5)
hold on
plot(u,z2,'LineWidth',1.5)
grid on
xlabel('Velocita'' vento [m/s]')
ylabel('Damping [%]')
zoom on
legend('Bending mode','Torsion mode')
axis([0 210 -40 40])


%% CALCOLO CON SMORZAMENTO AERODINAMICO

for i=1:length(u)
    B_s=[LL(i)*b^2./u(i)  0;(-LL(i)*b^2*(0.5+a)-LL(i)*d*b)./u(i) -LL(i)*d*C_media*M_aer./(4*u(i))]; %MATRICE DI SMORZAMENTO AERODINAMICO
    C_s=[0 LL(i)*b;0 -LL(i)*b*(0.5+a)-LL(i)*d]; %MATRICE DI RIGIDEZZA AERODINAMICA
    E_s=[kl*b^2 0;0 kt]; %MATRICE DI RIGIDEZZA STRUTTURALE
    D_s=zeros(2,2);
    F_s=[zeros(2,2) eye(2,2);-inv(M).*(C_s+E_s) -inv(M).*(B_s+D_s)];
    [vett_s val_s]=eig(F_s,eye(4,4));
    w1_s(i)=abs(val_s(1,1));  %AUTOVALORI
    w2_s(i)=abs(val_s(3,3));
    z1_s(i)=-real(val_s(1,1))./w1_s(i);
    z2_s(i)=-real(val_s(3,3))./w2_s(i);
end

figure()
subplot(3,1,1)
plot(u,w1_s,'LineWidth',1.5)
title('COMPORTAMENTO AERODINAMICO CON SMORZAMENTO')
hold on
plot(u,w2_s,'LineWidth',1.5)
grid on
ylabel('w [rad/s]')
axis([0 203 0 50])

subplot(3,1,2)
plot(u,w1_s/(2*pi),'LineWidth',1.5)
hold on
plot(u,w2_s/(2*pi),'LineWidth',1.5)
grid on
ylabel('Frequenza [Hz]')
legend('Bending mode','Torsion mode')
axis([0 203 0 8])

subplot(3,1,3)
plot(u,z1_s,'LineWidth',1)
hold on
plot(u,z2_s,'LineWidth',1)
grid on
xlabel('Velocita'' vento [m/s]')
ylabel('Damping [%]')
zoom on
axis([0 203 -0.2 1])


%% ANALISI D'ENERGIA

% MOTORE SPENTO (ce=0)

miu=0.04;
md=miu*m; %[kg] massa motore
beta=1/(1+miu);
eta=sqrt(3*miu/(8*(1+miu)));
cm=2*eta*beta*md*walp; %[N.s/m] Smorzamento totale motore
kd=0; % Rigidezza motore
cd=cm; % Smorzamento viscoso motore
d_m=0.1; % 0.09 [m] Presa motore. Distanza dall'asse elastico

M_m=[(m+md)*b^2 (Sa-md*d)*b;(Sa-md*d)*b I_ea+md*d^2]; %MATRICE DI MASSA

A_vett_m=[];
B_val_m=[];

for i=1:length(u)
    B_m=[LL(i)*b^2./u(i)  0;(-LL(i)*b^2*(0.5+a)-LL(i)*d*b)./u(i) -LL(i)*d*C_media*M_aer./(4*u(i))]; %MATRICE DI SMORZAMENTO AERODINAMICO
    C_m=[0 LL(i)*b;0 -LL(i)*b*(0.5+a)-LL(i)*d]; %MATRICE DI RIGIDEZZA AERODINAMICA
    E_m=[(kd+kl)*b^2 -kd*d_m;-kd*d_m kt+kd*d_m^2]; %MATRICE DI RIGIDEZZA STRUTTURALE
    D_m=[(cd)*b^2 -(cd)*d_m*b;-(cd)*d_m*b (cd)*d_m^2]; %MATRICE DI SMORZAMENTO STRUTTURALE
    F_m=[zeros(2,2) eye(2,2);-inv(M_m).*(C_m+E_m) -inv(M_m).*(B_m+D_m)];
    [vett_m val_m]=eig(F_m,eye(4,4));
    w1_m(i)=abs(val_m(1,1));
    w2_m(i)=abs(val_m(3,3));
    z1_m(i)=-real(val_m(1,1))./w1_m(i);
    z2_m(i)=-real(val_m(3,3))./w2_m(i);
    A_vett_m = cat(3, A_vett_m, vett_m);
    B_val_m = cat(3, B_val_m, val_m);
end

f1_m=w1_m/(2*pi);
f2_m=w2_m/(2*pi);

% VELOCITA FLUTTER

poss_m=find(z2_m<0);
pos_m=poss_m(1);
Vel_flutter_m=u(pos_m);
Frecuenza_flutter_m=f2_m(pos_m);
w1_flutter_m=w1_m(pos_m);
w2_flutter_m=w2_m(pos_m);
vett_flutter_m=A_vett_m(:,:,pos_m);
val_flutter_m=B_val_m(:,:,pos_m);

% MOTORE ACCESO

Ke=0.15; %[Nm/Arms] Cte motore
p=4; % Numero di poli 
R=1.4; %[ohm] Resistenza 
Rl=16.5; %[ohm] Resistenza di linea
L_m=8.5; %[mH] Induttanza
l=20e-3; %[m] Passo vite
ce=6*pi^2*Ke^2/((l^2)*(R+Rl)); % [N.s/m] Smorzamento elettrico motore

A_vett_m2=[];
B_val_m2=[];

for i=1:length(u)
    B_m2=[LL(i)*b^2./u(i)  0;(-LL(i)*b^2*(0.5+a)-LL(i)*d*b)./u(i) -LL(i)*d*C_media*M_aer./(4*u(i))]; %MATRICE DI SMORZAMENTO AERODINAMICO
    C_m2=[0 LL(i)*b;0 -LL(i)*b*(0.5+a)-LL(i)*d]; %MATRICE DI RIGIDEZZA AERODINAMICA
    E_m2=[(kd+kl)*b^2 -kd*d_m;-kd*d_m kt+kd*d_m^2]; %MATRICE DI RIGIDEZZA STRUTTURALE
    D_m2=[(ce+cd)*b^2 -(ce+cd)*d_m*b;-(ce+cd)*d_m*b (ce+cd)*d_m^2]; %MATRICE DI SMORZAMENTO STRUTTURALE
    F_m2=[zeros(2,2) eye(2,2);-inv(M_m).*(C_m2+E_m2) -inv(M_m).*(B_m2+D_m2)];
    [vett_m2 val_m2]=eig(F_m2,eye(4,4));
    w1_m2(i)=abs(val_m2(1,1));
    w2_m2(i)=abs(val_m2(3,3));
    z1_m2(i)=-real(val_m2(1,1))./w1_m2(i);
    z2_m2(i)=-real(val_m2(3,3))./w2_m2(i);
    A_vett_m2 = cat(3, A_vett_m2, vett_m2);
    B_val_m2= cat(3, B_val_m2, val_m2);
end

f1_m2=w1_m2/(2*pi);
f2_m2=w2_m2/(2*pi);

% VELOCITA FLUTTER

poss_m2=find(z2_m2<0);
pos_m2=poss_m2(1);
Vel_flutter_m2=u(pos_m2);
Frecuenza_flutter_m2=f2_m2(pos_m2);
w1_flutter_m2=w1_m2(pos_m2);
w2_flutter_m2=w2_m2(pos_m2);
vett_flutter_m2=A_vett_m2(:,:,pos_m2);
val_flutter_m2=B_val_m2(:,:,pos_m2);
val_potenza_m2=B_val_m2(:,:,pos_m);
vett_potenza_m2=A_vett_m2(:,:,pos_m);
Frecuenza_potenza_2_m2=f2_m2(pos_m);
Frecuenza_potenza_1_m2=f1_m2(pos_m);

% SOLUZIONE

phi_o_m=[0;10*pi/180;0;0];
A_m=inv(A_vett_m2(:,:,pos_m))*phi_o_m;
Eta_m=[];
Y_m=[];

    for t=0:0.01:800
        eta_m=[A_m(1,1)*exp(val_potenza_m2(1,1)*t); A_m(2,1)*exp(val_potenza_m2(2,2)*t); A_m(3,1)*exp(val_potenza_m2(3,3)*t); A_m(4,1)*exp(val_potenza_m2(4,4)*t)];
        y_m=vett_potenza_m2*eta_m;
        Eta_m = cat(3, Eta_m, eta_m);
        Y_m = cat(3, Y_m, y_m);
        h_m=Y_m(1,:);
        alp_m=Y_m(2,:); %alp in rad
        h_punto_m=Y_m(3,:); % Velocità lineare asse elastico
        alp_punto_m=Y_m(4,:); % Velocità angolare asse elastico
    end

h_presa_m=h_m-d_m*alp_m; % Spostamento punto di presa
h_punto_presa_m=h_punto_m-d_m*alp_punto_m; % Velocita punto di presa
teta_punto_m=2*pi*h_punto_presa_m/l; %[rad/s]
P_gen=(3*Rl*(Ke.*teta_punto_m).^2)/(2*(R+Rl)^2); 
P_gen(P_gen<0)=0;
t=[0:0.01:800];
E_gen=cumtrapz(t,P_gen);

% Forza elettromagnetica prodotta dal motore 

for i=1:length(h_punto_presa_m)
    if i==1
        h_dpunto_presa_m(i)=h_punto_presa_m(i)/t(i);
    else
        h_dpunto_presa_m(i)=(h_punto_presa_m(i)-h_punto_presa_m(i-1))/(t(i)-t(i-1));
    end
end

ft_m=md.*h_dpunto_presa_m+ce.*h_punto_presa_m; %[N] Forza elettromagnetica prodotta dal motore 

% Risultati

figure()
subplot(3,1,1)
plot(u,w1_m,'LineWidth',1.5)
title('COMPORTAMENTO DEL PROFILO CON SMORZAMENTO VISCOSO')
hold on
plot(u,w2_m,'LineWidth',1.5)
grid on
ylabel('w [rad/s]')


subplot(3,1,2)
plot(u,f1_m,'LineWidth',1.5)
hold on
plot(u,f2_m,'LineWidth',1.5)
grid on
ylabel('Frequenza [Hz]')


subplot(3,1,3)
plot(u,z1_m,'LineWidth',1.5)
hold on
plot(u,z2_m,'LineWidth',1.5)
grid on
xlabel('Velocita'' vento [m/s]')
ylabel('Damping [%]')
zoom on
hold on
legend('Bending mode','Torsion mode','Flutter')

figure()
subplot(3,1,1)
plot(u,w1_m2,'LineWidth',1.5)
title('COMPORTAMENTO DEL PROFILO CON MOTORE ELETTRICO')
hold on
plot(u,w2_m2,'LineWidth',1.5)
grid on
ylabel('w [rad/s]')


subplot(3,1,2)
plot(u,f1_m2,'LineWidth',1.5)
hold on
plot(u,f2_m2,'LineWidth',1.5)
grid on
ylabel('Frequenza [Hz]')


subplot(3,1,3)
plot(u,z1_m2,'LineWidth',1.5)
hold on
plot(u,z2_m2,'LineWidth',1.5)
grid on
xlabel('Velocita'' vento [m/s]')
ylabel('Damping [%]')
zoom on
hold on
plot(u(pos_m2),z2_m2(pos_m2),'s','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',8)
legend('Bending mode','Torsion mode','Flutter')


figure()
plot(u,z1_m,'b','LineWidth',1.5)
hold on
plot(u,z1_m2,'r','LineWidth',1.5)
hold on
plot(u,z2_m,'b','LineWidth',1.5)
hold on
plot(u,z2_m2,'r','LineWidth',1.5)
grid on
xlabel('Velocita'' vento [m/s]')
ylabel('Damping [%]')
hold on
plot(u(pos_m),z2_m(pos_m),'o','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',5)
plot(u(pos_m),z2_m2(pos_m),'o','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',5)
legend('Motore spento','Motore acceso')
zoom on

figure()
plot(t,h_m,'LineWidth',1.5)
grid on
title('Spostamento verticale asse elastico')
ylabel('h [m]')
xlabel('Tempo dopo Flutter [s]')


figure()
plot(t,h_presa_m,'LineWidth',1.5)
grid on
title('Spostamento verticale punto di presa')
ylabel('h_p_r_e_s_a [m]')
xlabel('Tempo dopo Flutter [s]')


alp_m=alp_m*180/pi; % alp in gradi
figure()
plot(t,alp_m,'LineWidth',1.5)
grid on
title('Spostamento angolare')
ylabel('Alpha [°]')
xlabel('Tempo dopo Flutter [s]')


figure()
plot(t,ft_m,'LineWidth',1.5)
grid on
title('Forza elettromagnetica prodotta dal motore  ')
ylabel('F_t [N]')
xlabel('Tempo dopo Flutter [s]')

figure()
plot(t,P_gen,'LineWidth',1.5)
grid on
title('Potenza generata')
ylabel('P [W]')
xlabel('Tempo dopo Flutter [s]')

figure()
plot(t,E_gen,'LineWidth',1.5)
grid on
title('Energia recuperabile')
ylabel('E [J]')
xlabel('Tempo dopo Flutter [s]')

